import 'dart:async';
import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:http/http.dart' as http;
import 'package:http/http.dart';
import 'package:intl/intl.dart';
import 'package:overlay_support/overlay_support.dart';
import 'package:sghi_core/afya_moja_core/afya_moja_core.dart';
import 'package:sghi_core/afya_moja_core/src/app_asset_strings.dart';
import 'package:sghi_core/afya_moja_core/src/app_strings.dart';
import 'package:sghi_core/app_wrapper/app_wrapper_base.dart';
import 'package:sghi_core/flutter_graphql_client/constants.dart';
import 'package:sghi_core/flutter_graphql_client/i_flutter_graphql_client.dart';

/// Formats a phone number by adding the country code and applying specific rules.
///
/// This function takes a `countryCode` and a `phoneNumber` as input and returns a
/// formatted phone number string. If the `countryCode` does not start with '+',
/// the function adds the '+' prefix. For country code '+1' (United States), the
/// formatted phone number includes only the `phoneNumber` without any changes.
/// For other country codes, the function removes leading '0' from the `phoneNumber`.
///
/// @param countryCode The country code associated with the phone number.
/// @param phoneNumber The phone number to be formatted.
/// @return A formatted phone number string.
String formatPhoneNumber({
  required String countryCode,
  required String phoneNumber,
}) {
  final String kCountryCode =
      !countryCode.startsWith('+') ? '+$countryCode' : countryCode;

  if (kCountryCode == '+1') {
    return '$countryCode$phoneNumber';
  }

  final String kPhoneNumber =
      phoneNumber.startsWith('0') ? phoneNumber.substring(1) : phoneNumber;

  return '$kCountryCode$kPhoneNumber';
}

/// expiresIn is in seconds
DateTime getTokenExpiryTimestamp(String? expiresIn) {
  final DateTime now = DateTime.now();
  if (expiresIn != null && expiresIn.isNotEmpty && isNumeric(expiresIn)) {
    return now.add(Duration(seconds: int.tryParse(expiresIn) ?? 0));
  }
  return now;
}

/// Checks if token has expired based on expiry date
/// returns true if token expires in 10 minutes or less
/// otherwise returns false
bool hasTokenExpired(DateTime expiresAt, DateTime now) {
  return expiresAt.difference(now).inMinutes < 10;
}

bool isNumeric(String? s) {
  if (s == null) {
    return false;
  }
  return int.tryParse(s) != null;
}

/// Extracts an error code from a JSON response body.
///
/// This function takes a JSON response body as input and attempts to extract
/// an error code. If the input body is not null and contains a key named 'code',
/// the value associated with the 'code' key is extracted as a string. The extracted
/// string is then parsed to an integer using `int.tryParse`. If parsing is successful,
/// the integer value is returned. If parsing fails or the 'code' key is not present,
/// a default error code of 11 is returned.
///
/// @param body The JSON response body from which to extract the error code.
/// @return An integer error code extracted from the JSON response body, or a default value of 11.
int extractErrorCode(Map<String, dynamic>? body) {
  if (body != null && body.containsKey('code')) {
    final String code = body['code']?.toString() ?? '';
    return int.tryParse(code) ?? 11;
  } else {
    return 11;
  }
}

/// Processes an HTTP response and returns a ProcessedResponse object.
///
/// This function takes an HTTP response as input and evaluates its status code.
/// If the status code indicates success (200, 201, or 202), a `ProcessedResponse`
/// object is returned with the 'ok' field set to `true`. For other status codes,
/// the response body is decoded as JSON, and an error `ProcessedResponse` object
/// is returned. The `extractErrorCode` function is used to extract an error code,
/// and the `getUserFriendlyMsg` and `getErrorMessage` functions are used to generate
/// error messages based on the error code and status code.
///
/// @param response The HTTP response to be processed.
/// @return A ProcessedResponse object containing the processed information.
ProcessedResponse processHttpResponse(Response response) {
  if (<int>[200, 201, 202].contains(response.statusCode)) {
    return ProcessedResponse(ok: true, response: response, code: 200);
  }

  final Map<String, dynamic> body =
      json.decode(response.body) as Map<String, dynamic>;

  final int code = extractErrorCode(body);

  if (response.statusCode == 400 || response.statusCode == 500) {
    return ProcessedResponse(
      ok: false,
      response: response,
      message: getUserFriendlyMsg(code),
      code: code,
    );
  }

  if (response.statusCode == 408) {
    return ProcessedResponse(
      ok: false,
      response: response,
      message: getUserFriendlyMsg(0),
      code: 0,
    );
  }

  return ProcessedResponse(
    ok: false,
    response: response,
    message: getErrorMessage(),
    code: code,
  );
}

/// Retrieves a user-friendly error message based on an error code.
///
/// This function takes an error code as input and returns a user-friendly
/// error message associated with that code. It uses a switch-case structure
/// to map error codes to corresponding messages. If the provided code does
/// not match any known error code, a default error message is returned.
///
/// @param code The error code for which to retrieve a user-friendly message.
/// @return A user-friendly error message corresponding to the provided error code.
String getUserFriendlyMsg(int code) {
  switch (code) {
    case 0:
      return slowInternet;
    case 4:
      return userWithThatPhoneExists;
    case 5:
      return userWithThatEmailExists;
    case 6:
      return usernameExists;
    case 7:
    case 10:
      return userNotFound;
    case 8:
      return wrongLoginCredentials;
    case 9:
      return pinNotFound;
    case 48:
      return pinExpired;
    case 72:
      return pendingPINResetRequest;
    case 73:
      return tooManyAttemptsString;

    default:
      return defaultUserFriendlyMessage;
  }
}

/// Retrieves an error message based on an optional custom message.
///
/// This function takes an optional custom error message as input and returns an
/// error message based on that input. If the provided custom message is empty,
/// the function returns a default error message. Otherwise, it constructs and
/// returns an error message that includes the custom message along with a generic
/// error statement. This function is useful for generating informative error
/// messages for various scenarios.
///
/// @param message An optional custom error message describing the error scenario.
/// @return An error message based on the provided custom message, or a default message.
String getErrorMessage([String message = '']) {
  if (message.isEmpty) {
    return defaultUserFriendlyMessage;
  } else {
    return 'Sorry, an error occurred while $message.'
        ' Please try again later, or contact support center';
  }
}

/// [isValidPhoneNumber] checks if a number is either a [Kenyan] , [American],  [UK] or [Belgium] phone number
bool isValidPhoneNumber(String phone) {
  if (kenyanPhoneRegExp.hasMatch(phone) ||
      (americanPhoneRegExp.hasMatch(phone)) ||
      (unitedKingdomRegExp.hasMatch(phone)) ||
      (genericInternationalRegExp.hasMatch(phone))) {
    return true;
  } else {
    return false;
  }
}

/// [validateKenyanNumber] checks if a number is either has either prefix of '+254111', '+254110', '+254100', '+2540101', '+254102' or '+2547xx'
bool validateKenyanNumber(String phone) {
  String prefix;
  final String fifthChar = phone.substring(4, 5);
  if (fifthChar == '7') {
    return true;
  } else if (fifthChar == '1') {
    prefix = phone.substring(0, 7);
    if (validPrefixes.contains(prefix)) {
      return true;
    }
    return false;
  } else {
    return false;
  }
}

/// Parses an error message from a JSON response body.
///
/// This function takes a JSON response body as input and attempts to parse an
/// error message from it. It looks for the presence of 'errors' or 'error' keys
/// in the input body and extracts the corresponding error value. If an error
/// message is found, it is returned as a `String`. If no error message is found,
/// or if the parsing process encounters unexpected data types, `null` is returned.
///
/// @param body The JSON response body from which to parse an error message.
/// @return A parsed error message as a `String`, or `null` if no error message is found.
String? parseError(Map<String, dynamic>? body) {
  if (body == null) return null;

  final Object? error =
      body.containsKey('errors') ? body['errors'] : body['error'];

  if (error == null) return null;

  if (error is List<dynamic>) {
    final Map<String, dynamic> firstEntry = error.first as Map<String, dynamic>;
    return firstEntry['message'] as String;
  }

  if (error is String) {
    return error.contains(RegExp('ID token', caseSensitive: false))
        ? kLoginLogoutPrompt
        : error;
  }

  return null;
}

/// [snackbar]
SnackBar snackbar({
  required dynamic
      content, // [content] must be either of type [Widget] or [String]
  int durationSeconds = 10,
  String? label,
  Function? callback,
}) {
  return SnackBar(
    content: content.runtimeType == String
        ? Text(content as String)
        : content as Widget,
    duration: Duration(seconds: durationSeconds),
    action: callback != null
        ? SnackBarAction(label: label!, onPressed: callback as void Function())
        : null,
  );
}

/// [Dismiss snackbar]
SnackBarAction dismissSnackBar(String text, Color color, BuildContext context) {
  return SnackBarAction(
    label: text,
    textColor: color,
    onPressed: () {
      ScaffoldMessenger.of(context).hideCurrentSnackBar();
    },
  );
}

/// [extractNamesInitials] => Extracts name initials from a name
///
/// Usage:
///
/// If you pass in a name like 'Abiud Orina', it returns 'AO'
String extractNamesInitials({required String name}) {
  final List<String> parts = name.trim().split(' ');

  // Remove parts that are empty
  parts.removeWhere((String p) => p.isEmpty);

  if (parts.isEmpty) {
    return 'UU';
  } else if (parts.length >= 2) {
    final StringBuffer initials = StringBuffer();
    for (int i = 0; i <= 1; i++) {
      final String part = parts[i];
      initials.write(part[0].toUpperCase());
    }
    return initials.toString().substring(0, 2);
  }
  return parts.first.split('')[0].toUpperCase();
}

/// [titleCase] returns a title cased sentence
String titleCase(String sentence) {
  return sentence
      .toLowerCase()
      .split(' ')
      .map((String e) => e.trim())
      .map((String word) => toBeginningOfSentenceCase(word))
      .join(' ');
}

/// [removeUnderscores] removes underscores from a sentence
String removeUnderscores(String sentence) {
  return titleCase(sentence.replaceAll('_', ' ').toLowerCase());
}

/// [Generic Fetch Function]
/// a generic fetch function for fetching all the problems, allergies
/// medications, tests and diagnoses for the current patient
/// in an episode
///
/// it takes in a [String queryString], the Map of the query variables [variables],
/// the BuildContext [context], and a stream controller [streamController] in which the data is added to
///
/// it then updates the stream controller with the returned data (if any) or
/// an error if there was an error
Future<void> genericFetchFunction({
  required StreamController<dynamic> streamController,
  required BuildContext context,
  required String queryString,
  required Map<String, dynamic> variables,
  required String logTitle,
  String? logDescription,
}) async {
  streamController.add(<String, dynamic>{'loading': true});

  final IGraphQlClient kClient = AppWrapperBase.of(context)!.graphQLClient;

  /// fetch the data from the api
  final http.Response response = await kClient.query(
    queryString,
    variables,
  );

  final Map<String, dynamic> payLoad = kClient.toMap(response);
  final String? error = parseError(payLoad);

  if (error != null) {
    return streamController
        .addError(<String, dynamic>{'error': kClient.parseError(payLoad)});
  }

  return (payLoad['data'] != null)
      ? streamController.add(payLoad['data'])
      : streamController.add(null);
}

/// [getDeviceType]
/// /// Determines the type of device screen based on its width and orientation.
///
/// This function takes a `BuildContext` as input and determines the type of the
/// device screen based on its width and orientation. It utilizes the `MediaQuery`
/// to extract information about the device's orientation and screen size. The
/// function calculates the device width and then classifies it as 'Desktop',
/// 'Tablet', or 'Mobile' based on specific width thresholds. The determined
/// `DeviceScreensType` is returned.
///
/// @param context The build context for accessing device-related information.
/// @return The type of device screen based on width and orientation.
DeviceScreensType getDeviceType(BuildContext context) {
  final MediaQueryData mediaQuery = MediaQuery.of(context);
  final Orientation deviceOrientation = mediaQuery.orientation;
  double deviceWidth = 0;
  if (deviceOrientation == Orientation.landscape) {
    deviceWidth = mediaQuery.size.height;
  } else {
    deviceWidth = mediaQuery.size.width;
  }

  if (deviceWidth > 1200) {
    return DeviceScreensType.Desktop;
  }
  if (deviceWidth > 600) {
    return DeviceScreensType.Tablet;
  }
  return DeviceScreensType.Mobile;
}

/// [getGreetingMessage] gets the current time of day and determines which type of greetings to show
/// to the user
String getGreetingMessage(String? firstName, {int? currentHour}) {
  final int hour = currentHour ?? DateTime.now().hour;
  final String name = firstName == null ? '' : ', $firstName';
  if (hour < 12) {
    return 'Good Morning$name';
  }
  if (hour < 17) {
    return 'Good Afternoon$name';
  }
  return 'Good Evening$name';
}

/// This function performs validation on a user-entered PIN value. It ensures that
/// the PIN is not empty, consists of only digits, and has a minimum length of four.
/// The function returns an error message string if the validation fails, otherwise
/// it returns `null` to indicate a valid PIN.
///
/// @param val The user-entered PIN value to be validated.
/// @return An error message string if validation fails, otherwise `null`.
String? userPinValidator(dynamic val) {
  final String value = val as String;
  if (value.isEmpty) {
    return 'A PIN is required';
  }
  if (!RegExp(r'^-?[0-9]+$').hasMatch(value)) {
    return 'Only digits are allowed, 0-9';
  }
  if (value.length < 4) {
    return 'Enter a four digit PIN';
  }
  return null;
}

/// Validates a user-entered CCC number.
///
/// This function performs validation on a user-entered CCC number. It ensures that
/// the CCC number is not empty and has a length between 7 and 15 characters. The
/// function returns an error message string if the validation fails, otherwise it
/// returns `null` to indicate a valid CCC number.
///
/// @param val The user-entered CCC number to be validated.
/// @return An error message string if validation fails, otherwise `null`.
String? cccNumberValidator(dynamic val) {
  final String value = val as String;
  if (value.isEmpty) {
    return 'Your CCC number is required';
  }
  if (value.length < 7 || value.length > 15) {
    return 'Enter a valid CCC number';
  }
  return null;
}

/// Converts a JSON role string to a corresponding RoleValue enumeration.
///
/// This function takes a JSON role string as input and converts it to the
/// corresponding `RoleValue` enumeration. If the input string is null, empty,
/// or matches the constant `UNKNOWN`, the function returns the default role
/// `RoleValue.CONTENT_MANAGEMENT`. Otherwise, it compares the lowercase names
/// of the `RoleValue` enum values with the lowercase input string to find a match.
/// The first matching `RoleValue` enum is returned. If no match is found, an
/// exception will be thrown.
///
/// @param roleString The JSON role string to convert to a `RoleValue` enum.
/// @return A `RoleValue` enum corresponding to the input role string.
RoleValue roleValueFromJson(String? roleString) {
  if (roleString == null || roleString.isEmpty || roleString == UNKNOWN) {
    return RoleValue.CONTENT_MANAGEMENT;
  }

  return RoleValue.values.firstWhere((RoleValue role) {
    return describeEnum(role)
            .toLowerCase()
            .compareTo(roleString.toLowerCase()) ==
        0;
  });
}

/// Converts a JSON gender string to a corresponding Gender enumeration.
///
/// This function takes a JSON gender string as input and converts it to the
/// corresponding `Gender` enumeration. If the input string is null, empty,
/// or matches the constant `UNKNOWN`, the function returns the default gender
/// `Gender.unknown`. Otherwise, it searches through the `Gender` enum values
/// and compares their lowercase names with the lowercase input string to find
/// a match. The first matching `Gender` enum is returned. If no match is found,
/// an exception will be thrown.
///
/// @param genderString The JSON gender string to convert to a `Gender` enum.
/// @return A `Gender` enum corresponding to the input gender string.
Gender genderFromJson(String? genderString) {
  if (genderString == null || genderString.isEmpty || genderString == UNKNOWN) {
    return Gender.unknown;
  }
  return Gender.values.where((Gender gender) {
    return gender.name.toLowerCase() == genderString.toLowerCase();
  }).first;
}

/// Converts a Gender enumeration to a JSON string representation.
///
/// This function takes a `Gender` enumeration as input and converts it to a JSON
/// string representation. If the input `Gender` is null, the function returns
/// the JSON string representation of the default gender `Gender.unknown`. Otherwise,
/// it returns the JSON string representation of the input gender's `name` property.
///
/// @param gender The Gender enumeration to convert to a JSON string.
/// @return A JSON string representation of the input Gender enumeration.
String genderToJson(Gender? gender) {
  return gender?.name ?? Gender.unknown.name;
}

/// Summarizes a given date/time string into a human-readable format.
///
/// This function takes a loaded date/time string as input and summarizes it into
/// a human-readable format, such as "X mins ago", "Y hours ago", or "Yesterday".
/// It calculates the time difference between the input date and the current date,
/// and generates appropriate summaries based on the time elapsed. If the date
/// parsing fails, the current date is used as a fallback. The function also uses
/// the `formatDate` function to provide a detailed date format when needed.
///
/// @param loadedDate The date/time string to summarize.
/// @return A human-readable summarized format of the input date/time.
String summarizeDate(String loadedDate) {
  final DateTime parsedDate =
      DateTime.tryParse(loadedDate)?.toLocal() ?? DateTime.now();
  final int difference = DateTime.now().difference(parsedDate).inMinutes;

  if (difference < 1) {
    return '0 mins ago';
  } else if (difference == 1) {
    return '1 min ago';
  } else if (difference < 60) {
    return '$difference mins ago';
  } else if (difference < 1440) {
    return '${(difference / 60).round()} hours ago';
  } else if (difference < 2880) {
    return 'Yesterday';
  }
  return formatDate(loadedDate);
}

/// Formats a given date/time string into a user-friendly date representation.
///
/// This function takes a date/time string as input and formats it into a
/// user-friendly date representation. It allows the option to show the time
/// component by setting the `showTime` parameter to `true`. The function
/// parses the input date string, and if parsing fails, it uses the current
/// date as a fallback. The formatted date includes the day, month, year,
/// and optionally the time component.
///
/// @param date The date/time string to format.
/// @param showTime Whether to include the time component in the formatted date.
/// @return A user-friendly formatted date string.
String formatDate(String date, {bool showTime = false}) {
  final DateTime parsedDate =
      DateTime.tryParse(date)?.toLocal() ?? DateTime.now();
  final String postDayTime = DateFormat.jm().format(parsedDate);
  final String postDay = DateFormat.d().format(parsedDate);
  final String postMonth = DateFormat.MMMM().format(parsedDate);
  final String postYear = DateFormat.y().format(parsedDate);
  return '$postDay $postMonth, $postYear${showTime ? ' at $postDayTime' : ''}';
}

/// Converts a duration in seconds to a formatted audio time representation.
///
/// This function takes a duration in seconds as input and converts it into a
/// formatted audio time representation (minutes:seconds). It calculates the
/// minutes and seconds components of the duration and formats the seconds
/// with a leading '0' if needed. The resulting formatted time string is returned.
///
/// @param duration The duration in seconds to convert to a formatted audio time.
/// @return A formatted audio time representation (minutes:seconds).
String audioTime(int duration) {
  Duration(seconds: duration);
  final int min = (duration / 60).floor();
  final int sec = duration % 60;
  String formattedSec;
  if (sec <= 9) {
    formattedSec = '0$sec';
  } else {
    formattedSec = '$sec';
  }

  return '$min:$formattedSec';
}

String contentReadDuration(int duration) =>
    duration == 0 ? '...' : '${(duration / 60).floor()} min read';
String contentAudioVideoDuration(int duration) =>
    duration == 0 ? '...' : '${(duration / 60).floor()} min';

// Parses date then converts it to the format 18 May 2021 at 12:00 AM
Widget humanizeDate({
  required TextStyle dateTextStyle,
  required String loadedDate,
  bool showTime = false,
  bool showYear = true,
}) {
  if (loadedDate != UNKNOWN && loadedDate.isNotEmpty) {
    final DateTime parsedDate =
        DateTime.tryParse(loadedDate)?.toLocal() ?? DateTime.now();

    final String dayTime = DateFormat.jm().format(parsedDate);
    final String day = DateFormat.d().format(parsedDate);
    final String month = DateFormat.MMM().format(parsedDate);
    final String year = DateFormat.y().format(parsedDate);

    return Row(
      children: <Widget>[
        Text(
          '$day $month ${showYear ? year : ''}${showTime ? ' at $dayTime' : ''}',
          style: dateTextStyle,
        ),
      ],
    );
  }

  return const SizedBox();
}

/// Displays a heads-up notification overlay.
///
/// This function displays a heads-up notification overlay with the provided
/// `title` and `body` text. The overlay consists of a container containing an
/// icon, title, and body text, displayed in a row layout. The notification
/// overlay is designed to be displayed at the top of the screen within a safe
/// area. The overlay will automatically disappear after a duration of 5 seconds.
///
/// @param title The title text of the notification overlay.
/// @param body The body text of the notification overlay.
/// @return An `OverlaySupportEntry` representing the displayed notification overlay.
OverlaySupportEntry headsUpNotification(String? title, String? body) {
  return showOverlayNotification(
    (BuildContext context) {
      return SafeArea(
        child: Container(
          width: double.infinity,
          margin: const EdgeInsets.symmetric(vertical: 8, horizontal: 8),
          padding: const EdgeInsets.all(10),
          decoration: BoxDecoration(
            color: Colors.white,
            borderRadius: BorderRadius.circular(8.0),
          ),
          child: Row(
            children: <Widget>[
              Container(
                width: 60,
                height: 60,
                padding: const EdgeInsets.all(10.0),
                margin: const EdgeInsets.only(right: 8),
                decoration: BoxDecoration(
                  color: Theme.of(context).primaryColor.withOpacity(0.2),
                  borderRadius: BorderRadius.circular(8.0),
                ),
                child: SvgPicture.asset(
                  notificationIcon,
                  colorFilter: ColorFilter.mode(
                      Theme.of(context).primaryColor, BlendMode.srcIn),
                ),
              ),
              Expanded(
                child: DefaultTextStyle(
                  style: normalSize13Text(greyTextColor),
                  child: Column(
                    mainAxisAlignment: MainAxisAlignment.center,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      Text(
                        title ?? '',
                        style: boldSize15Text(blackColor),
                        overflow: TextOverflow.ellipsis,
                        maxLines: 1,
                      ),
                      Text(
                        body ?? '',
                        maxLines: 2,
                        overflow: TextOverflow.ellipsis,
                      ),
                    ],
                  ),
                ),
              ),
            ],
          ),
        ),
      );
    },
    duration: const Duration(seconds: 5),
  );
}

/// Generates a title for a new chat message notification.
///
/// This function generates a title for a new chat message notification based
/// on the provided `username` and `channelName`. The title is constructed by
/// combining the base message "New message" with the `username` and `channelName`
/// if they are provided. The resulting title string provides information about
/// the source of the new chat message.
///
/// @param username The username of the sender of the new chat message.
/// @param channelName The name of the chat channel where the new message was sent.
/// @return A title for the new chat message notification.
String newChatMessageTitle(String? username, String? channelName) {
  String baseMessage = 'New message';

  if (username != null) {
    baseMessage = '$baseMessage from $username';
  }

  if (channelName != null) {
    baseMessage = '$baseMessage in $channelName';
  }

  return baseMessage;
}

/// Converts a contact type string to a ContactType enumeration value.
///
/// This function takes a contact type string as input and converts it into a
/// corresponding `ContactType` enumeration value. It checks if the input
/// `contactString` is null, empty, or equal to 'UNKNOWN', and if so, it returns
/// `ContactType.UNKNOWN`. Otherwise, it uses a `where` clause to search through
/// the `ContactType.values` enumeration and find a match based on the `name`
/// property, case-insensitively. The function then returns the matching `ContactType`
/// value. If no match is found, it returns `ContactType.UNKNOWN`.
///
/// @param contactString The contact type string to convert.
/// @return A `ContactType` enumeration value corresponding to the input string.
ContactType contactTypeFromJson(String? contactString) {
  if (contactString == null ||
      contactString.isEmpty ||
      contactString == UNKNOWN) {
    return ContactType.UNKNOWN;
  }
  return ContactType.values.where((ContactType contactType) {
    return contactType.name.toLowerCase() == contactString.toLowerCase();
  }).first;
}

/// Converts a consent status string to a ConsentStatus enumeration value.
///
/// This function takes a consent status string as input and converts it into a
/// corresponding `ConsentStatus` enumeration value. It checks if the input
/// `consentString` is null, empty, or equal to 'UNKNOWN', and if so, it returns
/// `ConsentStatus.UNKNOWN`. Otherwise, it uses the `firstWhere` method to search
/// through the `ConsentStatus.values` enumeration and find a match based on the
/// `name` property in a case-insensitive manner. If a match is found, the matching
/// `ConsentStatus` value is returned. If no match is found, it returns `ConsentStatus.UNKNOWN`.
///
/// @param consentString The consent status string to convert.
/// @return A `ConsentStatus` enumeration value corresponding to the input string.
ConsentStatus consentStatusFromJson(String? consentString) {
  if (consentString == null ||
      consentString.isEmpty ||
      consentString == UNKNOWN) {
    return ConsentStatus.UNKNOWN;
  }

  return ConsentStatus.values.firstWhere(
    (ConsentStatus consentStatus) {
      return consentStatus.name == consentString.toUpperCase();
    },
    orElse: () => ConsentStatus.UNKNOWN,
  );
}

/// Converts a ConsentStatus enumeration value to a consent status string for JSON serialization.
///
/// This function takes a `ConsentStatus` enumeration value as input and converts it into
/// a corresponding consent status string for JSON serialization. If the input `consentStatus`
/// is null, the function returns the string representation of `ConsentStatus.PENDING`.
/// If the input `consentStatus` is `ConsentStatus.UNKNOWN`, the function also returns
/// the string representation of `ConsentStatus.PENDING`. For other valid `ConsentStatus`
/// values, the function returns the string representation of the input value.
///
/// @param consentStatus The `ConsentStatus` enumeration value to convert.
/// @return A consent status string for JSON serialization.
String consentStatusToJson(ConsentStatus? consentStatus) {
  switch (consentStatus) {
    case null:
      return ConsentStatus.PENDING.name;
    case ConsentStatus.UNKNOWN:
      return ConsentStatus.PENDING.name;
    default:
      return consentStatus.name;
  }
}

DayOfWeek dayOfWeekFromJson(String? dayOfWeek) {
  if (dayOfWeek == null || dayOfWeek.isEmpty || dayOfWeek == UNKNOWN) {
    return DayOfWeek.UNKNOWN;
  }
  return DayOfWeek.values.where((DayOfWeek day) {
    return day.name.toLowerCase() == dayOfWeek.toLowerCase();
  }).first;
}
