// ignore_for_file: avoid_classes_with_only_static_members

import 'dart:convert';
import 'dart:math';

import 'package:http/http.dart';
import 'package:sghi_core/communities/core/endpoints.dart';
import 'package:sghi_core/communities/models/room_power_level.dart';
import 'package:sghi_core/communities/models/room_user.dart';
import 'package:sghi_core/communities/models/strings.dart';
import 'package:sghi_core/communities/models/sync_params.dart';
import 'package:sghi_core/communities/models/user.dart';
import 'package:sghi_core/flutter_graphql_client/constants.dart';
import 'package:sghi_core/flutter_graphql_client/i_flutter_graphql_client.dart';

class ChatAPI {
  /// Creates a Matrix room with the specified parameters.
  ///
  /// The [roomName] is the name of the room to be created.
  /// The [topic] is the room's topic that can be used to describe the room's purpose.
  /// The [client] is an instance of [IGraphQlClient] used to make the API call.
  ///
  /// Returns a [Response] object representing the API response for the room creation.
  ///
  /// Example:
  /// ```dart
  /// final roomName = 'My Awesome Room';
  /// final topic = 'This is a room for awesome discussions!';
  /// final client = IGraphQlClient();
  ///
  /// final Response response = await createRoom(
  ///   roomName: roomName,
  ///   topic: topic,
  ///   client: client,
  /// );
  ///
  // / // Use the response to handle the result of the room creation.
  /// ```
  static Future<Response> createRoom({
    required String roomName,
    required String topic,
    required IGraphQlClient client,
  }) async {
    final String alias = roomName.replaceAll(' ', '');

    final Map<String, dynamic> variables = <String, dynamic>{
      'visibility': 'private',
      'is_direct': false,
      'preset': 'private_chat',
      'invite': <String>[
        // Add a list of users to invite to this group
      ],
      'name': roomName,
      'room_alias_name': alias,
      'topic': topic,
    };

    final Response response = await client.callRESTAPI(
      endpoint: createRoomEndpoint(client.endpoint),
      method: kPostMethodName,
      variables: variables,
    );

    return response;
  }

  /// Checks whether a user with the specified [userID] can delete a message in a Matrix room based on their power level.
  ///
  /// The [roomID] is the unique identifier of the room where the message is located.
  /// The [userID] is the unique identifier of the user whose permission is being checked.
  /// The [senderID] is the unique identifier of the user who sent the message to be deleted.
  /// The [client] is an instance of [IGraphQlClient] used to make the API call.
  ///
  /// Returns a [Future<bool>] indicating whether the user can delete the message or not.
  ///
  /// Example:
  /// ```dart
  /// final roomID = 'your_room_id';
  /// final userID = 'your_user_id';
  /// final senderID = 'sender_user_id';
  /// final client = IGraphQlClient();
  ///
  /// final bool canDelete = await canDeleteMessage(
  ///   roomID: roomID,
  ///   userID: userID,
  ///   senderID: senderID,
  ///   client: client,
  /// );
  ///
  /// if (canDelete) {
  ///   // Allow the user to delete the message.
  /// } else {
  ///   // User does not have permission to delete the message.
  /// }
  /// ```
  static Future<bool> canDeleteMessage({
    required String roomID,
    required String userID,
    required String senderID,
    required IGraphQlClient client,
  }) async {
    final RoomPowerLevel powerLevels =
        await fetchRoomPowerLevels(roomID, client);

    final Map<String, dynamic>? powerLevelUser = powerLevels.users;

    final int? userLevel = powerLevelUser?[userID] as int?;

    // If they are the sender of the message or they are a moderator
    return senderID == userID || (userLevel != null && userLevel >= 50);
  }

  /// Deletes a room message with the specified [eventID] in a Matrix room.
  ///
  /// The [roomID] is the unique identifier of the room where the message is located.
  /// The [eventID] is the unique identifier of the message event to be deleted.
  /// The [senderID] is the unique identifier of the user who sent the message to be deleted.
  /// The [userID] is the unique identifier of the user trying to delete the message.
  /// The [client] is an instance of [IGraphQlClient] used to make the API call.
  ///
  /// Returns a [Future<Response>] representing the response from the API call.
  ///
  /// Example:
  /// ```dart
  /// final roomID = 'your_room_id';
  /// final eventID = 'your_message_event_id';
  /// final senderID = 'sender_user_id';
  /// final userID = 'your_user_id';
  /// final client = IGraphQlClient();
  ///
  /// final Response response = await deleteMessage(
  ///   roomID: roomID,
  ///   eventID: eventID,
  ///   senderID: senderID,
  ///   userID: userID,
  ///   client: client,
  /// );
  ///
  /// if (response.success) {
  ///   // Message deletion was successful.
  /// } else {
  ///   // Message deletion failed.
  /// }
  /// ```
  static Future<Response> deleteMessage({
    required String roomID,
    required String eventID,
    required String senderID,
    required String userID,
    required IGraphQlClient client,
  }) async {
    // Generate a temporary transaction ID to use in the API call
    final String tempTxnID = Random.secure().nextInt(1 << 32).toString();

    // Call the API to delete the message using the specified [eventID]
    final Response response = await client.callRESTAPI(
      endpoint: deleteMessageEventEndpoint(
          roomID, tempTxnID, eventID, client.endpoint),
      method: kPutMethodName,
      variables: <String, dynamic>{},
    );

    return response;
  }

  /// Returns the processed URL of a matrix media image from the provided [mediaUrl].
  ///
  /// This function converts a Matrix Content URI (MXC URI) to an actual image URL that can be used to
  /// display the image.
  ///
  /// The [mediaUrl] is the Matrix Content URI (MXC URI) of the media image.
  ///
  /// Returns a [String] representing the processed image URL, or `null` if [mediaUrl] is `null` or empty.
  ///
  /// Example:
  /// ```dart
  /// final mediaUrl = 'mxc://example.com/your_media_id';
  ///
  /// final String? imageUrl = getMediaURL(mediaUrl: mediaUrl);
  /// if (imageUrl != null) {
  ///   // Use the [imageUrl] to display the media image.
  /// } else {
  ///   // Invalid or empty [mediaUrl]. Cannot display the media image.
  /// }
  /// ```
  static String? getMediaURL({
    required String? mediaUrl,
    required String mediaServerUrl,
    required String baseUrl,
  }) {
    // Return `null` if the provided [mediaUrl] is `null` or empty
    if (mediaUrl?.isEmpty ?? true) {
      return null;
    }

    // Extract the media ID from the MXC URI
    final String? mediaID = mediaUrl?.split('/').last;

    // If the media ID is not empty, return the processed image URL using the media ID
    if (mediaID?.isNotEmpty ?? false) {
      return fetchMediaEndpoint(mediaID!, mediaServerUrl, baseUrl);
    }

    // If the media ID is empty or null, return `null`
    return null;
  }

  /// Synchronizes events from a HomeServer based on the provided [syncParams].
  ///
  /// This function retrieves events from the HomeServer and synchronizes them based on the specified
  /// [syncParams]. The [syncParams] parameter is of type [SyncParams] and contains the synchronization
  /// parameters such as `fullSync`, `since`, and `timeout`.
  ///
  /// The `fullSync` parameter determines whether a full synchronization should be performed or not.
  /// If `fullSync` is `true`, a full state synchronization will be performed. Otherwise, a partial
  /// synchronization will be done.
  ///
  /// The [syncParams.client] is the GraphQL client used to make the API call to the HomeServer.
  ///
  /// Returns a [Response] object representing the response from the HomeServer after synchronization.
  /// The response contains the synchronized events and other relevant data.
  ///
  /// Example:
  /// ```dart
  /// final syncParams = SyncParams(
  ///   fullSync: true,
  ///   since: '2023-05-01T00:00:00Z',
  ///   timeout: 30000,
  ///   client: myGraphQLClient,
  /// );
  ///
  /// final Response response = await synchronizeEvents(syncParams);
  /// if (response.statusCode == 200) {
  ///   // Synchronization successful, process the events in the response.
  /// } else {
  ///   // Synchronization failed, handle the error.
  /// }
  /// ```
  ///
  /// Note: The timeout for the synchronization request is set based on the value of `fullSync`.
  /// For a full synchronization, the timeout is set to 180 seconds, and for a partial synchronization,
  /// the timeout is set to 60 seconds.
  static Future<Response> synchronizeEvents(SyncParams syncParams) async {
    // Determine if a full sync should be performed or not
    final bool fullSync = syncParams.fullSync ?? false;

    // Build the endpoint URL with the required parameters
    String endpoint = syncEndpoint(syncParams.client.endpoint);

    // Add url params for full_state and since
    endpoint += '?full_state=$fullSync';
    endpoint += (syncParams.since != null && syncParams.since != UNKNOWN)
        ? '&since=${syncParams.since}'
        : '';
    endpoint += '&timeout=${syncParams.timeout}';

    // If fullSync is true, add a filter to include all types of events in the synchronization
    endpoint += fullSync
        ? '&filter={"room":{"state": {"lazy_load_members":true, "types":["*"]}, "timeline":{"lazy_load_members":true}}}'
        : '';

    // Get the GraphQL client from the syncParams
    final IGraphQlClient processedClient = syncParams.client;

    // Make the API call to the HomeServer and wait for the response
    final Response response = await processedClient.callRESTAPI(
      endpoint: endpoint,
      method: kGetMethodName,
      variables: <String, dynamic>{},
    ).timeout(Duration(seconds: fullSync ? 180 : 60));

    // Return the response from the HomeServer after synchronization
    return response;
  }

  /// Synchronizes events for a single room based on the provided [roomID].
  ///
  /// This function retrieves events for a specific room identified by the [roomID] and synchronizes
  /// them with the client's current state. The [roomID] parameter is the unique identifier of the
  /// room for which events will be synchronized.
  ///
  /// The [client] is the GraphQL client used to make the API call to the HomeServer.
  ///
  /// Returns a [Response] object representing the response from the HomeServer after synchronization.
  /// The response contains the synchronized events and other relevant data.
  ///
  /// Example:
  /// ```dart
  /// final String roomId = 'your_room_id_here';
  /// final IGraphQlClient myGraphQLClient = MyGraphQLClient();
  ///
  /// final Response response = await syncRoom(client: myGraphQLClient, roomID: roomId);
  /// if (response.statusCode == 200) {
  ///   // Synchronization successful, process the events in the response.
  /// } else {
  ///   // Synchronization failed, handle the error.
  /// }
  /// ```
  static Future<Response> syncRoom({
    required IGraphQlClient client,
    required String roomID,
  }) async {
    // Build the endpoint URL with the required parameters
    final String endpoint =
        '${syncEndpoint(client.endpoint)}??full_state=true&filter={"room":{"rooms":["$roomID"]}}';

    // Make the API call to the HomeServer and wait for the response
    final Response response = await client.callRESTAPI(
      endpoint: endpoint,
      method: kGetMethodName,
      variables: <String, dynamic>{},
    );

    // Return the response from the HomeServer after synchronization
    return response;
  }

  /// Fetches the power levels for a Matrix room based on the provided [roomID].
  ///
  /// This function retrieves the power levels for a specific room identified by the [roomID]
  /// and returns a [RoomPowerLevel] object containing the parsed power level data.
  ///
  /// The [client] is the GraphQL client used to make the API call to the HomeServer.
  ///
  /// Returns a [Future] that completes with the [RoomPowerLevel] object representing the power levels
  /// for the specified room.
  ///
  /// Example:
  /// ```dart
  /// final String roomId = 'your_room_id_here';
  /// final IGraphQlClient myGraphQLClient = MyGraphQLClient();
  ///
  /// final RoomPowerLevel powerLevels = await fetchRoomPowerLevels(roomID: roomId, client: myGraphQLClient);
  /// // Access power level information using powerLevels object.
  /// ```
  static Future<RoomPowerLevel> fetchRoomPowerLevels(
    String roomID,
    IGraphQlClient client,
  ) async {
    // Make the API call to the HomeServer to fetch power levels data for the specified room
    final Response response = await client.callRESTAPI(
      endpoint: roomPowerLevelsEndpoint(roomID, client.endpoint),
      method: kGetMethodName,
      variables: <String, dynamic>{},
    );

    // Decode the response body and parse it into a RoomPowerLevel object
    final dynamic data = await json.decode(utf8.decode(response.bodyBytes));
    final RoomPowerLevel roomPowerLevel =
        RoomPowerLevel.fromJson(data as Map<String, dynamic>);

    // Return the parsed RoomPowerLevel object
    return roomPowerLevel;
  }

  /// Fetches a list of all members of a Matrix room based on the provided [roomID].
  ///
  /// This function retrieves the list of all members of a specific room identified by the [roomID]
  /// and returns a [List<RoomUser>] containing information about each room member, such as their
  /// power level, moderator status, and user information.
  ///
  /// The [client] is the GraphQL client used to make the API call to the HomeServer.
  ///
  /// Returns a [Future] that completes with the [List<RoomUser>] representing the room members.
  ///
  /// Example:
  /// ```dart
  /// final String roomId = 'your_room_id_here';
  /// final IGraphQlClient myGraphQLClient = MyGraphQLClient();
  ///
  /// final List<RoomUser> roomMembers = await fetchRoomMembers(roomID: roomId, client: myGraphQLClient);
  /// // Access room members information using roomMembers list.
  /// ```
  static Future<List<RoomUser>> fetchRoomMembers({
    required String roomID,
    required IGraphQlClient client,
  }) async {
    // Generate the API URL to fetch room members
    final String url = joinedMembersEndpoint(roomID, client.endpoint);

    // Fetch room power levels for the specified room
    final RoomPowerLevel powerLevels =
        await fetchRoomPowerLevels(roomID, client);
    final Map<String, dynamic>? powerLevelUser = powerLevels.users;

    // Make the API call to the HomeServer to fetch room members data
    final Response response = await client.callRESTAPI(
      endpoint: url,
      method: kGetMethodName,
      variables: <String, dynamic>{},
    );

    // Decode the response body and extract the members data
    final Map<String, dynamic> decodedResponse =
        json.decode(response.body) as Map<String, dynamic>;
    final Map<String, dynamic> members =
        decodedResponse['joined'] as Map<String, dynamic>;

    // Map members data into RoomUser objects and add power level information
    final Map<dynamic, RoomUser> users =
        Map<dynamic, dynamic>.from(members).map(
      (dynamic key, dynamic value) {
        final Map<String, dynamic> v = value as Map<String, dynamic>;
        int? userLevel = 0;

        if (powerLevelUser?.containsKey(key) ?? false) {
          userLevel = powerLevelUser?[key.toString()] as int;
        }

        return MapEntry<dynamic, RoomUser>(
          key,
          RoomUser(
            powerLevel: userLevel,
            moderator: userLevel >= 50,
            roomUser: User(
              userID: key.toString(),
              avatarUri: v['avatar_url'] as String?,
              displayName: v['display_name'] as String?,
            ),
          ),
        );
      },
    );

    // Convert the Map values into a List and return the list of RoomUser objects
    return users.values.toList();
  }

  /// Invites a user with the provided [userID] into a Matrix room identified by [roomID].
  ///
  /// This function sends an invitation to the user with the specified [userID] to join the room
  /// specified by the [roomID]. The invitation is sent using the provided [client], which is the
  /// GraphQL client used to make the API call to the HomeServer.
  ///
  /// Returns a [Future] that completes with the [Response] received from the HomeServer after sending
  /// the invitation.
  ///
  /// Example:
  /// ```dart
  /// final String roomId = 'your_room_id_here';
  /// final String userId = 'user_id_to_invite';
  /// final IGraphQlClient myGraphQLClient = MyGraphQLClient();
  ///
  /// final Response inviteResponse = await inviteUsers(client: myGraphQLClient, roomID: roomId, userID: userId);
  /// // Handle the inviteResponse as needed.
  /// ```
  static Future<Response> inviteUsers({
    required IGraphQlClient client,
    required String roomID,
    required String userID,
  }) async {
    // Make the API call to invite the specified user to the room
    final Response response = await client.callRESTAPI(
      endpoint: inviteUserToRoomEndpoint(roomID, client.endpoint),
      method: kPostMethodName,
      variables: <String, dynamic>{'user_id': userID},
    );

    return response;
  }

  /// Joins a Matrix room specified by [roomID].
  ///
  /// This function sends a request to the HomeServer to join the Matrix room identified by the provided [roomID].
  /// The request is made using the provided [client], which is the GraphQL client used to make the API call to the HomeServer.
  ///
  /// Returns a [Future] that completes with the [Response] received from the HomeServer after attempting to join the room.
  ///
  /// Example:
  /// ```dart
  /// final String roomId = 'your_room_id_here';
  /// final IGraphQlClient myGraphQLClient = MyGraphQLClient();
  ///
  /// final Response joinResponse = await joinRoom(client: myGraphQLClient, roomID: roomId);
  /// // Handle the joinResponse as needed.
  /// ```
  static Future<Response> joinRoom({
    required IGraphQlClient client,
    required String roomID,
  }) async {
    // Make the API call to join the specified room
    final Response response = await client.callRESTAPI(
      endpoint: joinRoomEndpoint(roomID, client.endpoint),
      method: kPostMethodName,
      variables: <String, dynamic>{},
    );

    return response;
  }

  /// Removes a user with the specified [userID] from the Matrix room identified by [roomID].
  ///
  /// This function sends a request to the HomeServer to remove the specified user from the Matrix room.
  /// The request is made using the provided [client], which is the GraphQL client used to make the API call to the HomeServer.
  /// You can optionally provide a [reason] for the removal, which will be displayed as the reason for the user's removal in the room history.
  ///
  /// Returns a [Future] that completes with the [Response] received from the HomeServer after attempting to remove the user.
  ///
  /// Example:
  /// ```dart
  /// final String roomId = 'your_room_id_here';
  /// final String userIdToRemove = 'user_id_to_remove';
  /// final IGraphQlClient myGraphQLClient = MyGraphQLClient();
  ///
  /// final Response kickResponse = await kickUser(client: myGraphQLClient, roomID: roomId, userID: userIdToRemove);
  /// // Handle the kickResponse as needed.
  /// ```
  static Future<Response> kickUser({
    required IGraphQlClient client,
    required String roomID,
    required String userID,
    String? reason,
  }) async {
    // Make the API call to remove the specified user from the room
    final Response response = await client.callRESTAPI(
      endpoint: kickUserEndpoint(roomID, client.endpoint),
      method: kPostMethodName,
      variables: <String, dynamic>{
        'reason': reason ?? 'Removed by a moderator',
        'user_id': userID,
      },
    );

    return response;
  }

  /// Reports a message in a Matrix room.
  ///
  /// This function is used to mark a message as inappropriate and gives the message the lowest score of -100.
  /// It sends a request to the HomeServer to report the specified message in the Matrix room identified by [roomID].
  /// The message to be reported is identified by its [eventID].
  /// You can optionally provide a [reason] for the report, which will be displayed as the reason for reporting the message.
  ///
  /// Returns a [Future] that completes with the [Response] received from the HomeServer after attempting to report the message.
  ///
  /// Example:
  /// ```dart
  /// final String roomId = 'your_room_id_here';
  /// final String eventId = 'your_event_id_here';
  /// final IGraphQlClient myGraphQLClient = MyGraphQLClient();
  ///
  /// final Response reportResponse = await reportMessage(client: myGraphQLClient, roomID: roomId, eventID: eventId);
  /// // Handle the reportResponse as needed.
  /// ```
  static Future<Response> reportMessage({
    required IGraphQlClient client,
    required String roomID,
    required String eventID,
    String? reason,
  }) async {
    // Create the API endpoint URL for reporting the message
    final String url = reportMessageEndpoint(
        roomID: roomID, eventID: eventID, baseUrl: client.endpoint);

    // Make the API call to report the specified message in the room
    final Response response = await client.callRESTAPI(
      endpoint: url,
      method: kPostMethodName,
      variables: <String, dynamic>{
        'reason': reason ?? 'Inappropriate content',
        'score': -100,
      },
    );

    return response;
  }

  /// Fetches a list of all messages reported by users in a Matrix room.
  ///
  /// This function sends a request to the HomeServer to retrieve a list of all messages that have been reported by users in the Matrix room identified by [roomID].
  ///
  /// Returns a [Future] that completes with the [Response] received from the HomeServer, which contains the list of reported events.
  ///
  /// Example:
  /// ```dart
  /// final String roomId = 'your_room_id_here';
  /// final IGraphQlClient myGraphQLClient = MyGraphQLClient();
  ///
  /// final Response reportedEventsResponse = await fetchReportedEvents(client: myGraphQLClient, roomID: roomId);
  /// // Handle the reportedEventsResponse and extract the list of reported events as needed.
  /// ```
  static Future<Response> fetchReportedEvents({
    required IGraphQlClient client,
    required String roomID,
  }) async {
    // Create the API endpoint URL for fetching reported events
    final String url = reportedEventsEndpoint(roomID, client.endpoint);

    // Make the API call to fetch the reported events in the room
    final Response response = await client.callRESTAPI(
      endpoint: url,
      method: kGetMethodName,
      variables: <String, dynamic>{},
    );

    return response;
  }

  /// Fetches more details of a reported message in a Matrix room.
  ///
  /// This function sends a request to the HomeServer to retrieve more details of a message report identified by [reportID].
  ///
  /// Returns a [Future] that completes with the [Response] received from the HomeServer, which contains the details of the reported message.
  ///
  /// Example:
  /// ```dart
  /// final String reportId = 'your_report_id_here';
  /// final IGraphQlClient myGraphQLClient = MyGraphQLClient();
  ///
  /// final Response messageReportResponse = await fetchMessageReport(client: myGraphQLClient, reportID: reportId);
  /// // Handle the messageReportResponse and extract the details of the reported message as needed.
  /// ```
  static Future<Response> fetchMessageReport({
    required IGraphQlClient client,
    required String reportID,
  }) async {
    // Create the API endpoint URL for fetching the message report
    final String url = eventReportByIDEndpoint(reportID, client.endpoint);

    // Make the API call to fetch the message report details
    final Response response = await client.callRESTAPI(
      endpoint: url,
      method: kGetMethodName,
      variables: <String, dynamic>{},
    );

    return response;
  }

  /// Deletes a report for a message in a Matrix room.
  ///
  /// This function sends a request to the HomeServer to delete a report for a message identified by [reportID].
  ///
  /// Returns a [Future] that completes with the [Response] received from the HomeServer, indicating the result of the deletion.
  ///
  /// Example:
  /// ```dart
  /// final String reportId = 'your_report_id_here';
  /// final IGraphQlClient myGraphQLClient = MyGraphQLClient();
  ///
  /// final Response deleteReportResponse = await deleteMessageReport(client: myGraphQLClient, reportID: reportId);
  /// // Handle the deleteReportResponse and check if the report was successfully deleted.
  /// ```
  static Future<Response> deleteMessageReport({
    required IGraphQlClient client,
    required String reportID,
  }) async {
    // Create the API endpoint URL for deleting the message report
    final String url = eventReportByIDEndpoint(reportID, client.endpoint);

    // Make the API call to delete the message report
    final Response response = await client.callRESTAPI(
      endpoint: url,
      method: kDeleteMethodName,
      variables: <String, dynamic>{},
    );

    return response;
  }

  /// Leaves a Matrix room.
  ///
  /// This function sends a request to the HomeServer for the user to leave the room identified by [roomID].
  /// After leaving the room, the user will not be able to access it anymore.
  ///
  /// Returns a [Future] that completes with the [Response] received from the HomeServer, indicating the result of the leave action.
  ///
  /// Example:
  /// ```dart
  /// final String roomId = 'your_room_id_here';
  /// final IGraphQlClient myGraphQLClient = MyGraphQLClient();
  ///
  /// final Response leaveRoomResponse = await leaveRoom(client: myGraphQLClient, roomID: roomId);
  /// // Handle the leaveRoomResponse and check if the user successfully left the room.
  /// ```
  static Future<Response> leaveRoom({
    required IGraphQlClient client,
    required String roomID,
  }) async {
    // Create the API endpoint URL for leaving the room
    final String url = leaveRoomEndpoint(roomID, client.endpoint);

    // Make the API call to leave the room
    final Response response = await client.callRESTAPI(
      endpoint: url,
      method: kPostMethodName,
      variables: <String, dynamic>{},
    );

    return response;
  }

  /// Searches for members to add to a Matrix room.
  ///
  /// This function sends a request to the HomeServer to search for members based on the [searchParam].
  /// The search results will be limited to the [limit] specified (default is 10).
  ///
  /// Returns a [Future] that completes with the [Response] received from the HomeServer, containing the search results.
  ///
  /// Example:
  /// ```dart
  /// final String searchTerm = 'john';
  /// final int limit = 5;
  /// final IGraphQlClient myGraphQLClient = MyGraphQLClient();
  ///
  /// final Response searchResults = await searchGroupMembers(
  ///   client: myGraphQLClient,
  ///   searchParam: searchTerm,
  ///   limit: limit,
  /// );
  /// // Handle the searchResults and use the found members to add to the room.
  /// ```
  static Future<Response> searchGroupMembers({
    required IGraphQlClient client,
    required String searchParam,
    int limit = 10,
  }) async {
    // Create the API endpoint URL for searching group members
    final String endpoint = searchMembersEndpoint(client.endpoint);

    // Make the API call to search for members
    final Response response = await client.callRESTAPI(
      endpoint: endpoint,
      method: kPostMethodName,
      variables: <String, dynamic>{'limit': limit, 'search_term': searchParam},
    );

    return response;
  }

  /// Signs in to a Matrix home server using username and password.
  ///
  /// This function sends a request to the HomeServer to sign in the user with the provided [username] and [password].
  /// It uses the 'm.login.password' authentication type for the login process.
  ///
  /// Returns a [Future] that completes with the [Response] received from the HomeServer, containing the login result.
  ///
  /// Example:
  /// ```dart
  /// final String username = 'john_doe';
  /// final String password = 'securePassword';
  /// final IGraphQlClient myGraphQLClient = MyGraphQLClient();
  ///
  /// final Response loginResponse = await signIn(
  ///   client: myGraphQLClient,
  ///   username: username,
  ///   password: password,
  /// );
  /// // Handle the loginResponse and manage the user's session.
  /// ```
  static Future<Response> signIn({
    required IGraphQlClient client,
    required String? username,
    required String? password,
  }) async {
    // Define the authentication type for login
    const String loginAuthType = 'm.login.password';

    // Prepare the variables for the API call
    final Map<String, dynamic> variables = <String, dynamic>{
      'type': loginAuthType,
      'identifier': <String, dynamic>{'type': 'm.id.user', 'user': username},
      'password': password,
    };

    // Make the API call to sign in the user
    final Response response = await client.callRESTAPI(
      endpoint: loginEndpoint(client.endpoint),
      method: kPostMethodName,
      variables: variables,
    );

    return response;
  }

  /// Sends a message to a Matrix room.
  ///
  /// This function sends a message to the specified [roomID] using the provided [message].
  /// The message type is set to 'm.text', indicating a text-based message.
  ///
  /// Returns a [Future] that completes with the [Response] received from the HomeServer, containing the message sending result.
  ///
  /// Example:
  /// ```dart
  /// final String roomId = '!roomID:example.com';
  /// final String message = 'Hello, everyone!';
  /// final IGraphQlClient myGraphQLClient = MyGraphQLClient();
  ///
  /// final Response messageResponse = await sendMessage(
  ///   client: myGraphQLClient,
  ///   roomID: roomId,
  ///   message: message,
  /// );
  /// // Handle the messageResponse to confirm the message is sent successfully.
  /// ```
  static Future<Response> sendMessage({
    required IGraphQlClient client,
    required String roomID,
    required String message,
  }) async {
    // Generate a temporary transaction ID for the message
    final String tempTxnID = Random.secure().nextInt(1 << 32).toString();

    // Prepare the variables for the API call
    final Map<String, dynamic> variables = <String, dynamic>{
      'body': message,
      'msgtype': 'm.text',
    };

    // Make the API call to send the message
    final Response response = await client.callRESTAPI(
      endpoint: sendMessageEventEndpoint(roomID, tempTxnID, client.endpoint),
      method: kPutMethodName,
      variables: variables,
    );

    return response;
  }

  /// Sends a media message to a Matrix room.
  ///
  /// This function sends a media message to the specified [roomID] using the provided [messageText] and [imageMXCUrl].
  /// The message type is set to 'm.image' to indicate it contains an image.
  ///
  /// Returns a [Future] that completes with the [Response] received from the HomeServer, containing the message sending result.
  ///
  /// Example:
  /// ```dart
  /// final String roomId = '!roomID:example.com';
  /// final String message = 'Check out this image!';
  /// final String imageUrl = 'mxc://example.com/12345';
  /// final IGraphQlClient myGraphQLClient = MyGraphQLClient();
  ///
  /// final Response messageResponse = await sendMediaMessage(
  ///   client: myGraphQLClient,
  ///   roomID: roomId,
  ///   messageText: message,
  ///   imageMXCUrl: imageUrl,
  /// );
  /// // Handle the messageResponse to confirm the media message is sent successfully.
  /// ```
  static Future<Response> sendMediaMessage({
    required IGraphQlClient client,
    required String roomID,
    required String messageText,
    required String imageMXCUrl,
  }) async {
    final String tempTxnID = Random.secure().nextInt(1 << 32).toString();

    final Response response = await client.callRESTAPI(
      endpoint: sendMessageEventEndpoint(roomID, tempTxnID, client.endpoint),
      method: kPutMethodName,
      variables: <String, dynamic>{
        'body': messageText,
        'msgtype': 'm.image',
        'url': imageMXCUrl,
        'info': <String, dynamic>{
          'mimetype': 'image/jpeg',
          'size': 12345,
          'thumbnail_info': <String, dynamic>{
            'mimetype': 'image/jpeg',
            'size': 6789,
          },
          'thumbnail_url': imageMXCUrl,
          'w': 640,
          'h': 480,
          'external_info': <String, dynamic>{
            'caption': messageText,
          },
        },
      },
    );

    return response;
  }

  /// Promotes a user to moderator in a Matrix room.
  ///
  /// This function promotes the specified [userID] to a moderator (power level 50) in the room with the given [roomID].
  /// It fetches the current power levels in the room, adds the user to the moderator list, and then updates the room's power levels accordingly.
  ///
  /// Returns a [Future] that completes with the [Response] received from the HomeServer, containing the result of the promotion.
  ///
  /// Example:
  /// ```dart
  /// final String roomId = '!roomID:example.com';
  /// final String userId = '@john_doe:example.com';
  /// final IGraphQlClient myGraphQLClient = MyGraphQLClient();
  ///
  /// final Response promotionResponse = await promoteToModerator(
  ///   client: myGraphQLClient,
  ///   roomID: roomId,
  ///   userID: userId,
  /// );
  /// // Handle the promotionResponse to confirm the user is promoted successfully.
  /// ```
  static Future<Response> promoteToModerator({
    required IGraphQlClient client,
    required String roomID,
    required String userID,
  }) async {
    final RoomPowerLevel powerLevels =
        await fetchRoomPowerLevels(roomID, client);

    powerLevels.users?.addAll(<String, dynamic>{userID: 50});

    final Map<String, dynamic> powerLevelJson = powerLevels.toJson();

    final String endpoint =
        promoteUserToModeratorEndpoint(roomID, client.endpoint);

    final Response response = await client.callRESTAPI(
      endpoint: endpoint,
      method: kPutMethodName,
      variables: powerLevelJson,
    );

    return response;
  }

  /// Demotes a moderator to a user.
  ///
  /// This function demotes a moderator to a regular user by assigning a power level of 0 to the specified user within the room.
  ///
  /// Parameters:
  /// - `client`: An instance of [IGraphQlClient] used to make GraphQL requests.
  /// - `roomID`: The unique identifier of the room where the user is a moderator.
  /// - `userID`: The unique identifier of the user to be demoted from moderator to user.
  ///
  /// Returns:
  /// A [Future<Response>] representing the result of the demotion operation. The [Response] contains the response data from the server.
  ///
  /// Example Usage:
  /// ```dart
  /// final client = MyGraphQlClient(); // Replace MyGraphQlClient() with your own GraphQL client instance.
  /// final roomID = 'abc123'; // Replace 'abc123' with the actual room ID.
  /// final userID = 'def456'; // Replace 'def456' with the actual user ID.
  ///
  /// try {
  ///   final response = await demoteFromModerator(client: client, roomID: roomID, userID: userID);
  ///   print('Moderator demoted successfully: ${response.statusCode}');
  /// } catch (e) {
  ///   print('Error demoting moderator: $e');
  /// }
  /// ```
  static Future<Response> demoteFromModerator({
    required IGraphQlClient client,
    required String roomID,
    required String userID,
  }) async {
    // Fetch the current room power levels.
    final RoomPowerLevel powerLevels =
        await fetchRoomPowerLevels(roomID, client);

    // Set the user's power level to 0 (demote from moderator).
    powerLevels.users?.addAll(<String, dynamic>{userID: 0});

    // Convert the power levels to a JSON object.
    final Map<String, dynamic> powerLevelJson = powerLevels.toJson();

    // Define the API endpoint for promoting the user to moderator.
    final String endpoint =
        promoteUserToModeratorEndpoint(roomID, client.endpoint);

    // Make the REST API call to demote the user.
    final Response response = await client.callRESTAPI(
      endpoint: endpoint,
      method: kPutMethodName,
      variables: powerLevelJson,
    );

    return response;
  }

  /// Bans a user from a room.
  ///
  /// This function bans a user from a specific room, removing them from the room and preventing them from joining until they are unbanned.
  ///
  /// Parameters:
  /// - `client`: An instance of [IGraphQlClient] used to make GraphQL requests.
  /// - `roomID`: The unique identifier of the room from which the user will be banned.
  /// - `userID`: The unique identifier of the user to be banned.
  /// - `reason`: An optional reason for the ban. If not provided, a default reason will be used ("Banned by a moderator").
  ///
  /// Returns:
  /// A [Future<Response>] representing the result of the ban operation. The [Response] contains the response data from the server.
  ///
  /// Example Usage:
  /// ```dart
  /// final client = MyGraphQlClient(); // Replace MyGraphQlClient() with your own GraphQL client instance.
  /// final roomID = 'abc123'; // Replace 'abc123' with the actual room ID.
  /// final userID = 'def456'; // Replace 'def456' with the actual user ID.
  /// final banReason = 'Violation of community guidelines'; // Replace with the actual reason for the ban (optional).
  ///
  /// try {
  ///   final response = await banUser(client: client, roomID: roomID, userID: userID, reason: banReason);
  ///   print('User banned successfully: ${response.statusCode}');
  /// } catch (e) {
  ///   print('Error banning user: $e');
  /// }
  /// ```
  static Future<Response> banUser({
    required IGraphQlClient client,
    required String roomID,
    required String userID,
    String? reason,
  }) async {
    // Make the REST API call to ban the user from the room.
    final Response response = await client.callRESTAPI(
      endpoint: banUserEndpoint(roomID, client.endpoint),
      method: kPostMethodName,
      variables: <String, dynamic>{
        'reason': reason ?? 'Banned by a moderator',
        'user_id': userID,
      },
    );

    return response;
  }

  /// Lists all banned members for a specific room.
  ///
  /// This function fetches a list of all members who have been banned from the specified room.
  ///
  /// Parameters:
  /// - `roomID`: The unique identifier of the room for which to fetch the list of banned members.
  /// - `client`: An instance of [IGraphQlClient] used to make GraphQL requests.
  ///
  /// Returns:
  /// A [Future<Response>] representing the result of the API call. The [Response] contains the response data from the server, including the list of banned members.
  ///
  /// Example Usage:
  /// ```dart
  /// final client = MyGraphQlClient(); // Replace MyGraphQlClient() with your own GraphQL client instance.
  /// final roomID = 'abc123'; // Replace 'abc123' with the actual room ID.
  ///
  /// try {
  ///   final response = await fetchBannedMembers(roomID: roomID, client: client);
  ///   final List bannedMembers = response.data['bannedMembers'];
  ///   print('Banned Members: $bannedMembers');
  /// } catch (e) {
  ///   print('Error fetching banned members: $e');
  /// }
  /// ```
  static Future<Response> fetchBannedMembers({
    required String roomID,
    required IGraphQlClient client,
  }) async {
    // Make the REST API call to fetch the list of banned members.
    final Response response = await client.callRESTAPI(
      endpoint: bannedMembersEndpoint(roomID, client.endpoint),
      method: kGetMethodName,
      variables: <String, dynamic>{},
    );

    return response;
  }

  /// Updates the admin status of a user.
  ///
  /// This function updates the admin status of a user by specifying whether they should have admin privileges or not.
  ///
  /// Parameters:
  /// - `userID`: The unique identifier of the user for whom to update the admin status.
  /// - `admin`: A boolean value indicating whether the user should have admin privileges (true) or not (false).
  /// - `client`: An instance of [IGraphQlClient] used to make GraphQL requests.
  ///
  /// Returns:
  /// A [Future<Response>] representing the result of the API call. The [Response] contains the response data from the server, confirming the successful update of admin status.
  ///
  /// Example Usage:
  /// ```dart
  /// final client = MyGraphQlClient(); // Replace MyGraphQlClient() with your own GraphQL client instance.
  /// final userID = 'def456'; // Replace 'def456' with the actual user ID to update.
  /// final isAdmin = true; // Set this to true to give admin privileges, or false to remove admin privileges.
  ///
  /// try {
  ///   final response = await updateAdminStatus(
  ///     userID: userID,
  ///     password: password,
  ///     admin: isAdmin,
  ///     client: client,
  ///   );
  ///   print('Admin status updated successfully: ${response.statusCode}');
  /// } catch (e) {
  ///   print('Error updating admin status: $e');
  /// }
  /// ```
  static Future<Response> updateAdminStatus({
    required String userID,
    required bool admin,
    required IGraphQlClient client,
  }) async {
    // Make the REST API call to update the user's admin status.
    final Response response = await client.callRESTAPI(
      endpoint: updateAdminStatusEndpoint(userID, client.endpoint),
      method: kPutMethodName,
      variables: <String, dynamic>{'admin': admin},
    );

    return response;
  }

  /// Removes a ban placed on a user from a room.
  ///
  /// This function removes a ban placed on a specific user from a room, allowing the user to be invited and join the room again.
  ///
  /// Parameters:
  /// - `client`: An instance of [IGraphQlClient] used to make GraphQL requests.
  /// - `roomID`: The unique identifier of the room from which the user is to be unbanned.
  /// - `userID`: The unique identifier of the user to be unbanned.
  ///
  /// Returns:
  /// A [Future<Response>] representing the result of the unban operation. The [Response] contains the response data from the server.
  ///
  /// Example Usage:
  /// ```dart
  /// final client = MyGraphQlClient(); // Replace MyGraphQlClient() with your own GraphQL client instance.
  /// final roomID = 'abc123'; // Replace 'abc123' with the actual room ID.
  /// final userID = 'def456'; // Replace 'def456' with the actual user ID.
  ///
  /// try {
  ///   final response = await unBanUser(client: client, roomID: roomID, userID: userID);
  ///   print('User unbanned successfully: ${response.statusCode}');
  /// } catch (e) {
  ///   print('Error unbanning user: $e');
  /// }
  /// ```
  static Future<Response> unBanUser({
    required IGraphQlClient client,
    required String roomID,
    required String userID,
  }) async {
    final Response response = await client.callRESTAPI(
      endpoint: unBanUserEndpoint(roomID, client.endpoint),
      method: kPostMethodName,
      variables: <String, dynamic>{'user_id': userID},
    );

    return response;
  }
}
