import 'package:sghi_core/flutter_graphql_client/i_flutter_graphql_client.dart';

/// Params used to configure synchronization of Matrix events.
///
/// This class represents the parameters used to configure the synchronization of Matrix events.
/// It is typically used in the [synchronizeEvents] function.
///
/// Example usage:
/// ```dart
/// final syncParams = SyncParams(
///   client: graphQlClient,
///   accessToken: 'your_access_token',
///   fullState: false,
///   fullSync: false,
///   timeout: 10000,
///   since: 'your_since_token',
///   forceFull: false,
/// );
/// ```
/// Used in the [synchronizeEvents] function
class SyncParams {
  /// Creates an instance of [SyncParams].
  ///
  /// The [client] parameter represents the client used to synchronize the events.
  /// The [accessToken] parameter is a token that identifies the currently logged in user.
  /// The [fullState] parameter determines whether to fetch the full state of the server during the sync.
  /// The [fullSync] parameter determines whether a full sync is required.
  /// The [timeout] parameter specifies how long to keep the connection alive.
  /// The [since] parameter represents a point in time from where to return events from.
  /// The [forceFull] parameter determines whether to force a full sync.
  SyncParams({
    required this.client,
    this.accessToken,
    this.fullState = false,
    this.fullSync = false,
    this.timeout = 10000,
    this.since,
    this.forceFull = false,
  });

  /// A token that identifies the currently logged in user
  final String? accessToken;

  /// Whether to fetch the full state of the server during this sync
  final bool? fullState;

  /// Whether a full sync is required
  final bool? fullSync;

  /// How long to keep the connection alive. The API will keep returning events
  /// as long as they are available for this period
  final int? timeout;

  /// A point in time from where to return events from. Used for subsequent calls
  final String? since;

  /// Whether to force a full sync
  final bool? forceFull;

  /// The client to use to synchronize the events
  final IGraphQlClient client;
}
