import 'dart:async';
import 'dart:convert';

import 'package:gql/ast.dart' as ast;
import 'package:gql/language.dart' as lang;
import 'package:http/http.dart';
import 'package:sghi_core/flutter_graphql_client/constants.dart';
import 'package:source_span/source_span.dart';

enum ContentType { json, form }

class GraphQlClientCustom extends ICustomGraphQlClient {
  GraphQlClientCustom(
      String idToken, String endpoint, Map<String, String> requestHeaders) {
    super.idToken = idToken;
    super.endpoint = endpoint;
    super.requestHeaders = requestHeaders;
  }
}

/// [ICustomGraphQlClient] is the blueprint of a valid GraphQL client.
/// To construct a graphQL client, extend this class. This class provide
/// out of the box implementation of client methods but they can be overridden.
/// Best use case for overriding is when defining a mock client. Check [mocks.dart]
/// for a concrete mocking implementation.
abstract class ICustomGraphQlClient extends BaseClient {
  late String idToken;
  late String endpoint;
  late Map<String, String> requestHeaders;

  bool clientIsValid() {
    return true;
  }

  bool checkQueryString(String query) {
    try {
      // Parses graphql query string to check if its a valid graphql query
      final ast.DocumentNode _ = lang.parseString(
        query,
      );
      return true;
    } on SourceSpanException catch (_) {
      return false;
    }
  }

  Uri fromUriOrString(dynamic uri) =>
      uri is String ? Uri.parse(uri) : uri as Uri;

  Map<String, dynamic> toMap(Response? response) {
    if (response == null) return <String, dynamic>{};
    final dynamic _res = json.decode(utf8.decode(response.bodyBytes));
    if (_res is List<dynamic>) return _res[0] as Map<String, dynamic>;
    return _res as Map<String, dynamic>;
  }

  Future<Response> query(
    String queryString,
    Map<String, dynamic> variables, {
    Duration? timeout,
  }) async {
    if (!this.checkQueryString(queryString)) {
      return throw const FormatException(
          'The provided query string is malformed');
    }

    final Map<String, dynamic> bodyMap = <String, dynamic>{
      'query': queryString,
      'variables': variables,
    };

    return this.postWithTimeout(bodyMap, timeout: timeout);
  }

  Future<Response> postWithTimeout(
    Map<String, dynamic> bodyMap, {
    Duration? timeout,
  }) {
    return this
        .post(fromUriOrString(this.endpoint), body: json.encode(bodyMap))
        .timeout(
      timeout ?? const Duration(seconds: kRequestTimeoutSeconds),
      onTimeout: () {
        return Response(
          json.encode(kTimeoutResponsePayload),
          408,
          headers: this.requestHeaders,
        );
      },
    );
  }

  String? parseError(Map<String, dynamic>? body) {
    if (body == null) return null;

    final Object? error =
        body.containsKey('errors') ? body['errors'] : body['error'];

    if (error == null) return null;

    if (error is List<dynamic>) {
      final Map<String, dynamic> firstEntry =
          error.first as Map<String, dynamic>;
      return firstEntry['message'] as String;
    }

    if (error is String) {
      return error.contains(RegExp('ID token', caseSensitive: false))
          ? kLoginLogoutPrompt
          : error;
    }

    return null;
  }

  @override
  Future<StreamedResponse> send(BaseRequest request) async {
    request.headers.addAll(this.requestHeaders);
    return request.send();
  }
}
