// Dart imports:
import 'dart:async';
import 'dart:convert';

// Package imports:
import 'package:http/http.dart';
import 'package:sghi_core/flutter_graphql_client/constants.dart';

/// [AuthServerAPIClient] is the blueprint of a valid GraphQL client.
/// To construct a graphQL client, extend this class. This class provide
/// out of the box implementation of client methods but they can be overridden.
/// Best use case for overriding is when defining a mock client. Check [mocks.dart]
/// for a concrete mocking implementation.
abstract class AuthServerAPIClient extends BaseClient {
  late Map<String, String> requestHeaders;

  Uri fromUriOrString(dynamic uri) =>
      uri is String ? Uri.parse(uri) : uri as Uri;

  /// [callRESTAPI] is used when making unauthenticated REST backend calls
  Future<Response> callRESTAPI({
    required String endpoint,
    required String method,
    Map<String, String>? bodyFields,
    Map<String, dynamic>? variables,

    /// The timeout in seconds
    Duration? timeout,
    Map<String, String>? customHeaders,
  }) async {
    final Request request = Request(method, fromUriOrString(endpoint));

    if (bodyFields != null) {
      request.bodyFields = bodyFields;
    }
    if (variables != null) {
      request.body = json.encode(variables);
    }
    return Response.fromStream(
      await send(request).timeout(
        timeout ?? const Duration(seconds: kRequestTimeoutSeconds),
        onTimeout: () {
          final String timeoutInput = json.encode(kTimeoutResponsePayload);
          final List<int> body = utf8.encode(timeoutInput);
          return StreamedResponse(
            ByteStream.fromBytes(body),
            408,
            headers: requestHeaders,
          );
        },
      ),
    );
  }

  @override
  Future<StreamedResponse> send(BaseRequest request) async {
    request.headers.addAll(requestHeaders);
    return request.send();
  }
}
