@Timeout(Duration(seconds: 60))
library;

import 'dart:convert';

import 'package:flutter_test/flutter_test.dart';
import 'package:http/http.dart';
import 'package:sghi_core/afya_moja_core/src/domain/core/entities/processed_response.dart';
import 'package:sghi_core/afya_moja_core/src/helpers.dart';
import 'package:sghi_core/flutter_graphql_client/constants.dart';
import 'package:sghi_core/flutter_graphql_client/i_custom_graphql_client.dart';

import 'test_utils.dart';

void main() {
  group('FlutterGraphQlClientCustom', () {
    final GraphQlClientCustom validClient = GraphQlClientCustom(
      testToken,
      testURL,
      testRequestHeaders,
    );

    const String validGraphQLQuery = '''
          query UserInfo(\$id: ID!) {
            user(id: \$id) {
              id
              name
            }
          }
        ''';

    final Map<String, dynamic> validBody = <String, dynamic>{
      'id_token': null,
      'is_anonymous': true,
      'refresh_token': 'some-refresh-token',
      'uid': 'BkOpHPj9hLRnhYcmglIpc0VEQ9p1',
      'expires_in': '3600'
    };

    final List<Map<String, dynamic>> validBodyAsList = <Map<String, dynamic>>[
      <String, dynamic>{
        'id_token': null,
        'is_anonymous': true,
        'refresh_token': 'some-refresh-token',
        'uid': 'BkOpHPj9hLRnhYcmglIpc0VEQ9p1',
        'expires_in': '3600'
      }
    ];

    final Map<String, dynamic> idTokenErrorBody = <String, dynamic>{
      'errors': 'ID token error',
    };

    final Map<String, dynamic> errorBody = <String, dynamic>{
      'errors': 'generic error occurred',
    };

    final Map<String, dynamic> errorListBody = <String, dynamic>{
      'errors': <Map<String, dynamic>>[
        <String, String>{'message': 'generic list error occurred'}
      ]
    };
    final Response validResponse = Response(
      json.encode(
        validBody,
      ),
      201,
    );

    test('client should be valid if token and url are not null', () {
      final bool isValid = validClient.clientIsValid();
      expect(isValid, true);
    });

    test('should throw [FormatException] if query string is malformed',
        () async {
      expect(
        () async => validClient.query('invalid query', <String, dynamic>{
          'id': 'error',
        }),
        throwsA(isA<FormatException>()),
      );
    });

    test('should throw [Exception] if Variable param is not a valid map',
        () async {
      expect(
        () async => validClient.query(validGraphQLQuery, validBody),
        throwsA(isA<Exception>()),
      );
    });

    test('should if response is not a valid List<map>', () async {
      final Response resp = Response(json.encode(validBodyAsList), 200);
      final Map<String, dynamic> m = validClient.toMap(resp);
      expect(m, isA<Map<String, dynamic>>());
    });

    test('should return empty map from null response', () async {
      final Map<String, dynamic> m = validClient.toMap(null);
      expect(m, isA<Map<String, dynamic>>());
    });

    test(
        "Given a valid SIL GraphQL client's getters when called on a valid should the appropriate return values should be returned",
        () {
      expect(validClient.requestHeaders, testRequestHeaders);
    });

    test(
        'Given a valid SIL GraphQL client the toMap method should return a valid Map from the provided response',
        () {
      expect(validClient.toMap(validResponse), validBody);
    });

    test(
      'should show appropriate prompt or error depending on body supplied',
      () async {
        // valid case
        expect(validClient.parseError(validBody), null);
        // token error
        expect(validClient.parseError(idTokenErrorBody), kLoginLogoutPrompt);
        // other error(except token)
        expect(validClient.parseError(errorBody), 'generic error occurred');
        expect(validClient.parseError(errorListBody),
            'generic list error occurred');
      },
    );

    test('should timeout with json content type', () async {
      const String validGraphQLQuery = '''
        query UserInfo(\$id: ID!) {
          user(id: \$id) {
            id
            name
          }
        }
      ''';

      final Response response = await GraphQlClientCustom(
        testToken,
        testCustomURL,
        testRequestHeaders,
      ).query(
        validGraphQLQuery,
        <String, dynamic>{},
        timeout: const Duration(seconds: 1),
      );

      final ProcessedResponse processedResponse = processHttpResponse(response);

      final Map<String, dynamic> parsedResponse = jsonDecode(
        utf8.decode(processedResponse.response.bodyBytes),
      ) as Map<String, dynamic>;

      expect(parsedResponse['error'],
          'Network connection unreliable. Please try again later.');
    });
  });
}
